# ArcaneArts

## Cloning
To fetch all submodules, use the `--recursive` parameter:  
``git clone --recursive https://github.com/Mazi-S/cgue21-ArcaneArts``

To manually clone the necessary submodules, use:  
``git submodule update --init``

## Project Files
To generate the project files, run: `scripts/GenerateProjects.bat`

## Resources & Tutorials
### Libraries
* [entt](https://github.com/skypjack/entt) (entity-component-system)

* [physx](https://github.com/NVIDIAGameWorks/PhysX) (physics library)
  * [User Guide](https://gameworksdocs.nvidia.com/PhysX/4.1/documentation/physxguide/Index.html)
  * [API Reference](https://gameworksdocs.nvidia.com/PhysX/4.1/documentation/physxapi/files/index.html)

* [irrklang](https://www.ambiera.com/irrklang/index.html) (sound library)
  * [documentation](https://www.ambiera.com/irrklang/docu/index.html)

* [GLFW](https://github.com/glfw/glfw)
* [Glad](https://github.com/Dav1dde/glad)
* [glm](https://github.com/g-truc/glm) (mathematics library)
* [spdlog](https://github.com/gabime/spdlog) (logging library)
* [stb](https://github.com/nothings/stb) (image loader)
* [tinyobjloader](https://github.com/tinyobjloader/tinyobjloader) (obj loader)
* [yaml-cpp](https://github.com/jbeder/yaml-cpp) (YAML parser)
* [ImGui](https://github.com/ocornut/imgui) (GUI for debug/develop)

### Tutorials
* C++ / OpenGL
  * [YouTube - The Cherno](https://www.youtube.com/user/TheChernoProject)  
* Uniform Buffers
  * [lighthouse3d.com](https://www.lighthouse3d.com/tutorials/glsl-tutorial/uniform-blocks/)  
* Textures
  * [learnopengl.com](https://learnopengl.com/Getting-started/Textures)
* Shadow-Mapping
  * [opengl-tutorial.org](http://www.opengl-tutorial.org/intermediate-tutorials/tutorial-16-shadow-mapping/)
  * [learnopengl.com](https://learnopengl.com/Advanced-Lighting/Shadows/Shadow-Mapping)
* Particle System (using Transform Feedback)
  * [ogldev.atspace.co.uk](http://ogldev.atspace.co.uk/www/tutorial28/tutorial28.html)
* Cubemaps / Skyboxes / Environment Mapping
  * [learnopengl.com](https://learnopengl.com/Advanced-OpenGL/Cubemaps)
  * [YouTube - Brian Will](https://www.youtube.com/watch?v=QYvi1akO_Po)
* Normal Mapping
  * [learnopengl.com](https://learnopengl.com/Advanced-Lighting/Normal-Mapping)
* HDR
  * [learnopengl.com](https://learnopengl.com/Advanced-Lighting/HDR)
* Bloom
  * [learnopengl.com](https://learnopengl.com/Advanced-Lighting/Bloom)

### Models
* Beholder
  * "[Beholder](https://sketchfab.com/3d-models/beholder-2b28e79581d049958036cbac20b011a4)" by [Lekrieg](https://sketchfab.com/Lekrieg) is licensed under [CC Attribution](https://creativecommons.org/licenses/by/4.0/)
* Campfire
  * "[Lowpoly Campfire](https://sketchfab.com/3d-models/lowpoly-campfire-2b2fb40f34be47d2a710f8207f322ca5)" by [Dewberry](https://sketchfab.com/Dewberry) is licensed under [CC Attribution](https://creativecommons.org/licenses/by/4.0/)
* Cave
  * "[Cave](https://sketchfab.com/3d-models/cave-516a23277d3c485587e2bd14d245ac32)" by [inpakgames](https://sketchfab.com/inpakgames) is licensed under [CC Attribution](https://creativecommons.org/licenses/by/4.0/)
* Hands
  * "[Hand Topology Technical Demonstration](https://sketchfab.com/3d-models/hand-topology-technical-demonstration-686674f8bba44a54849cbefe63169a09)" by [lanternlit](https://sketchfab.com/lanternlit) is licensed under [CC Attribution](https://creativecommons.org/licenses/by/4.0/)
* House
  * "[House](https://sketchfab.com/3d-models/house-74797358830a47eb982a0cb69a732461)" by [makcutka250](https://sketchfab.com/makcutka250) is licensed under [CC Attribution](https://creativecommons.org/licenses/by/4.0/)
* Knight
  * "[Skull Knight](https://sketchfab.com/3d-models/skull-knight-e81fbf68b4c84f889f2b48df593daaa2)" by [iChill](https://sketchfab.com/mwisson) is licensed under [CC Attribution](https://creativecommons.org/licenses/by/4.0/)
* Pedestal
  * "[Pedestal](https://sketchfab.com/3d-models/pedestal-2074d31d216b417da713b7dda84f76cc)" by [apolllo6](https://sketchfab.com/apolllo6) is licensed under [CC Attribution](https://creativecommons.org/licenses/by/4.0/)
* Forest
  * "[Lowpoly Asset Collection | little nature pack](https://sketchfab.com/3d-models/lowpoly-asset-collection-little-nature-pack-765b6c119ad24940a029297094cb95c5)" by [DFS studio](https://sketchfab.com/DFS_studio) is licensed under [CC Attribution](https://creativecommons.org/licenses/by/4.0/)
* Clouds
  * "[Cloud 1 - Low poly](https://sketchfab.com/3d-models/cloud-1-low-poly-6acc8185b0ca4e9d909d6eb96229a85d)" by [lightguard](https://sketchfab.com/lightguard) is licensed under [CC Attribution](https://creativecommons.org/licenses/by/4.0/)
  * "[Cloud 2 - Low poly](https://sketchfab.com/3d-models/cloud-2-low-poly-5a7ba902db274dd8a7f7c184db04ebde)" by [lightguard](https://sketchfab.com/lightguard) is licensed under [CC Attribution](https://creativecommons.org/licenses/by/4.0/)
  * "[Cloud 2 - Low poly](https://sketchfab.com/3d-models/cloud-3-low-poly-aaf34de8ee2f42ca9e843d3b557266a7)" by [lightguard](https://sketchfab.com/lightguard) is licensed under [CC Attribution](https://creativecommons.org/licenses/by/4.0/)
